@@ -3,6 +3,7 @@ A Python library written in Rust to read from and write to Apache Arrow IPC (als
 and Apache Parquet.
 """
 from typing import Iterable, List, Optional
+import datetime
 
 import _arrowdantic_internal
 
@@ -68,6 +69,28 @@ class DataType:
     def large_string(cls) -> "DataType":
         return cls._from_type(_arrowdantic_internal.DataType.large_string())
 
+    @classmethod
+    def binary(cls) -> "DataType":
+        return cls._from_type(_arrowdantic_internal.DataType.binary())
+
+    @classmethod
+    def large_binary(cls) -> "DataType":
+        return cls._from_type(_arrowdantic_internal.DataType.large_binary())
+
+    @classmethod
+    def timestamp(cls, tz: Optional[datetime.tzinfo] = None) -> "DataType":
+        if tz:
+            tz = _format_offset(tz.utcoffset(None))
+        return cls._from_type(_arrowdantic_internal.DataType.timestamp(tz))
+
+    @classmethod
+    def date(cls) -> "DataType":
+        return cls._from_type(_arrowdantic_internal.DataType.date())
+
+    @classmethod
+    def time(cls) -> "DataType":
+        return cls._from_type(_arrowdantic_internal.DataType.time())
+
     @classmethod
     def _from_type(cls, dt: _arrowdantic_internal.DataType) -> "DataType":
         a = DataType()
@@ -194,6 +217,119 @@ class Int64Array(Array):
         self._array = _arrowdantic_internal.Int64Array(values)
 
 
+class TimestampArray(Int64Array):
+    """An array of 64-bit signed integers each representing a datetime with the same timezone"""
+
+    __slots__ = ("_tz",)
+
+    def __init__(self, values: List[Optional[datetime.datetime]]):
+        def _transform(value: Optional[datetime.datetime]):
+            if value is None:
+                return None
+            else:
+                seconds = value.timestamp()
+                microseconds = int(seconds * 10**6)
+                return microseconds
+
+        if not values:
+            tz = None
+            self._tz = None
+        else:
+            tz = values[0].timetz()
+            self._tz = tz.tzinfo
+            tz = _format_offset(tz.utcoffset())
+
+        values = list(map(_transform, values))
+        self._array = _arrowdantic_internal.Int64Array.from_ts_us(values, tz)
+
+    def __iter__(self):
+        return TimestampIterator(self._array.__iter__(), self._tz)
+
+
+class TimestampIterator:
+    __slots__ = ("_iter", "_tz")
+
+    def __init__(self, iter, tz):
+        self._iter = iter
+        self._tz = tz
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        dt_i64 = next(self._iter)
+        if dt_i64:
+            return datetime.datetime.fromtimestamp(dt_i64 / 10**6, self._tz)
+
+
+_DATE_EPOCH = datetime.datetime.utcfromtimestamp(0).date()
+
+class DateArray(Int32Array):
+    """An array of 32-bit signed integers each representing the day since epoch"""
+
+    def __init__(self, values: List[Optional[datetime.date]]):
+        def _transform(value: Optional[datetime.date]):
+            if value is None:
+                return None
+            else:
+                return (value - _DATE_EPOCH).days
+
+        values = list(map(_transform, values))
+        self._array = _arrowdantic_internal.Int32Array.from_date(values)
+
+    def __iter__(self):
+        return DateIterator(self._array.__iter__())
+
+
+class DateIterator:
+    __slots__ = ("_iter")
+
+    def __init__(self, iter):
+        self._iter = iter
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        dt_i32 = next(self._iter)
+        if dt_i32:
+            return _DATE_EPOCH + datetime.timedelta(days=dt_i32)
+
+
+
+class TimeArray(Int64Array):
+    """An array of 64-bit signed integers each representing the naive time since midnight with microsecond precision"""
+
+    def __init__(self, values: List[Optional[datetime.time]]):
+        def _transform(value: Optional[datetime.time]):
+            if value is None:
+                return None
+            else:
+                return ((value.hour * 60 + value.minute) * 60 + value.second) * 10**6 + value.microsecond
+
+        values = list(map(_transform, values))
+        self._array = _arrowdantic_internal.Int64Array.from_time_us(values)
+
+    def __iter__(self):
+        return TimeIterator(self._array.__iter__())
+
+
+class TimeIterator:
+    __slots__ = ("_iter")
+
+    def __init__(self, iter):
+        self._iter = iter
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        time_us = next(self._iter)
+        if time_us:
+            return datetime.datetime.fromtimestamp(time_us / 10**6).time()
+
+
+
 class Float32Array(Array):
     """An array of 32-bit floating point"""
 
@@ -462,3 +598,23 @@ class ODBCChunkIter:
 
     def __next__(self) -> Chunk:
         return Chunk._from_chunk(next(self._iter))
+
+
+def _format_offset(off):
+    # copied from std - it is private there :/
+    s = ""
+    if off is not None:
+        if off.days < 0:
+            sign = "-"
+            off = -off
+        else:
+            sign = "+"
+        hh, mm = divmod(off, datetime.timedelta(hours=1))
+        mm, ss = divmod(mm, datetime.timedelta(minutes=1))
+        s += "%s%02d:%02d" % (sign, hh, mm)
+        if ss or ss.microseconds:
+            s += ":%02d" % ss.seconds
+
+            if ss.microseconds:
+                s += ".%06d" % ss.microseconds
+    return s
