@@ -5,11 +5,11 @@ use arrow2::{
         Array, BinaryArray as _BinaryArray, BooleanArray as _BooleanArray, PrimitiveArray,
         Utf8Array,
     },
-    datatypes::PhysicalType,
+    datatypes::{DataType, PhysicalType, TimeUnit},
 };
 
-use pyo3::class::basic::CompareOp;
 use pyo3::prelude::*;
+use pyo3::{class::basic::CompareOp, types::PyType};
 
 use super::datatypes;
 use super::iterator;
@@ -75,11 +75,133 @@ primitive!(UInt32Array, UInt32Iterator, u32);
 primitive!(UInt64Array, UInt64Iterator, u64);
 primitive!(Int8Array, Int8Iterator, i8);
 primitive!(Int16Array, Int16Iterator, i16);
-primitive!(Int32Array, Int32Iterator, i32);
-primitive!(Int64Array, Int64Iterator, i64);
+//primitive!(Int32Array, Int32Iterator, i32);
+//primitive!(Int64Array, Int64Iterator, i64);
 primitive!(Float32Array, Float32Iterator, f32);
 primitive!(Float64Array, Float64Iterator, f64);
 
+#[derive(Clone, PartialEq, Debug)]
+#[pyclass]
+pub struct Int64Array(pub PrimitiveArray<i64>);
+
+#[pymethods]
+impl Int64Array {
+    #[new]
+    fn new(values: &PyAny) -> PyResult<Self> {
+        if let Ok(values) = values.extract::<Vec<i64>>() {
+            Ok(Self(PrimitiveArray::<i64>::from_vec(values)))
+        } else if let Ok(values) = values.extract::<Vec<Option<i64>>>() {
+            Ok(Self(PrimitiveArray::<i64>::from(values)))
+        } else {
+            todo!()
+        }
+    }
+
+    fn __repr__(&self) -> String {
+        format!("{:?}", &self.0 as &dyn Array)
+    }
+
+    fn __str__(&self) -> String {
+        self.__repr__()
+    }
+
+    fn __len__(&self) -> usize {
+        self.0.len()
+    }
+
+    fn __iter__(slf: PyRef<Self>) -> iterator::Int64Iterator {
+        iterator::Int64Iterator::new(slf)
+    }
+
+    #[getter(type)]
+    fn dtype(&self) -> datatypes::DataType {
+        datatypes::DataType(self.0.data_type().clone())
+    }
+
+    fn __richcmp__(&self, py: Python, other: PyObject, op: CompareOp) -> PyResult<bool> {
+        Ok(if let Ok(other) = other.extract::<Int64Array>(py) {
+            match op {
+                CompareOp::Eq => self.0 == other.0,
+                CompareOp::Ne => self.0 != other.0,
+                _ => todo!(),
+            }
+        } else {
+            false
+        })
+    }
+
+    #[classmethod]
+    fn from_ts_us(_: &PyType, values: &PyAny, tz: Option<String>) -> PyResult<Self> {
+        let values = Self::new(values)?;
+        Ok(Self(
+            values.0.to(DataType::Timestamp(TimeUnit::Microsecond, tz)),
+        ))
+    }
+
+    #[classmethod]
+    fn from_time_us(_: &PyType, values: &PyAny) -> PyResult<Self> {
+        let values = Self::new(values)?;
+        Ok(Self(values.0.to(DataType::Time64(TimeUnit::Microsecond))))
+    }
+}
+
+#[derive(Clone, PartialEq, Debug)]
+#[pyclass]
+pub struct Int32Array(pub PrimitiveArray<i32>);
+
+#[pymethods]
+impl Int32Array {
+    #[new]
+    fn new(values: &PyAny) -> PyResult<Self> {
+        if let Ok(values) = values.extract::<Vec<i32>>() {
+            Ok(Self(PrimitiveArray::<i32>::from_vec(values)))
+        } else if let Ok(values) = values.extract::<Vec<Option<i32>>>() {
+            Ok(Self(PrimitiveArray::<i32>::from(values)))
+        } else {
+            todo!()
+        }
+    }
+
+    fn __repr__(&self) -> String {
+        format!("{:?}", &self.0 as &dyn Array)
+    }
+
+    fn __str__(&self) -> String {
+        self.__repr__()
+    }
+
+    fn __len__(&self) -> usize {
+        self.0.len()
+    }
+
+    fn __iter__(slf: PyRef<Self>) -> iterator::Int32Iterator {
+        iterator::Int32Iterator::new(slf)
+    }
+
+    #[getter(type)]
+    fn dtype(&self) -> datatypes::DataType {
+        datatypes::DataType(self.0.data_type().clone())
+    }
+
+    fn __richcmp__(&self, py: Python, other: PyObject, op: CompareOp) -> PyResult<bool> {
+        Ok(if let Ok(other) = other.extract::<Int32Array>(py) {
+            match op {
+                CompareOp::Eq => self.0 == other.0,
+                CompareOp::Ne => self.0 != other.0,
+                _ => todo!(),
+            }
+        } else {
+            false
+        })
+    }
+
+    #[classmethod]
+    fn from_date(_: &PyType, values: &PyAny) -> PyResult<Self> {
+        let values = Self::new(values)?;
+        Ok(Self(values.0.to(DataType::Date32)))
+    }
+}
+
 #[derive(Clone, PartialEq, Debug)]
 #[pyclass]
 pub struct BooleanArray(pub _BooleanArray);
@@ -131,16 +253,6 @@ impl BooleanArray {
     }
 }
 
-macro_rules! primitive {
-    ($array:expr, $py:expr,$type:ty, $local:ident) => {{
-        let array = $array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<$type>>()
-            .unwrap();
-        $local(array.clone()).into_py($py)
-    }};
-}
-
 macro_rules! binary {
     ($name:ident, $iterator:ident, $type:ty) => {
         #[derive(Clone, PartialEq, Debug)]
@@ -257,6 +369,16 @@ macro_rules! string {
 string!(StringArray, StringIterator, i32);
 string!(LargeStringArray, LargeStringIterator, i64);
 
+macro_rules! primitive {
+    ($array:expr, $py:expr,$type:ty, $local:ident) => {{
+        let array = $array
+            .as_any()
+            .downcast_ref::<PrimitiveArray<$type>>()
+            .unwrap();
+        $local(array.clone()).into_py($py)
+    }};
+}
+
 pub fn to_py_object(py: Python, array: &dyn Array) -> PyObject {
     use arrow2::datatypes::PrimitiveType::*;
     match array.data_type().to_physical_type() {
